local aceHook = LibStub("AceHook-3.0")
local aceEvent = LibStub("AceEvent-3.0")
local _G = getfenv(0)
local format = _G.format
local UnitIsPlayer = _G.UnitIsPlayer
local UnitCanAttack = _G.UnitCanAttack
local UnitIsPVP = _G.UnitIsPVP
local UnitClass = _G.UnitClass
local UnitPlayerControlled = _G.UnitPlayerControlled
local UnitExists = _G.UnitExists
local UnitName = _G.UnitName
local GetGuildInfo = _G.GetGuildInfo
local UnitIsUnit = _G.UnitIsUnit
local UnitCreatureFamily = _G.UnitCreatureFamily
local UnitCreatureType = _G.UnitCreatureType
local UnitMana = _G.UnitMana
local UnitManaMax = _G.UnitManaMax
local UnitIsTapped = _G.UnitIsTapped
local UnitIsTappedByPlayer = _G.UnitIsTappedByPlayer

local friendlist = {}
local TipUpdateFrame = CreateFrame("Frame")
local function resetScripts()
	TipUpdateFrame:SetScript("OnUpdate", nil)
end

CreateFrame("GameTooltip", "tehtip", UIParent, "GameTooltipTemplate")
tehtip:SetOwner(WorldFrame, "ANCHOR_NONE")

local HealthBar_OnValueChanged_org = HealthBar_OnValueChanged
local function HealthBar_OnValueChanged_new (self, value, smooth)
	HealthBar_OnValueChanged_org()
	if ( not self.lockColor ) then
		self:SetStatusBarColor(0, 0, 1);
	end
end
HealthBar_OnValueChanged = HealthBar_OnValueChanged_new

GameTooltipStatusBar:SetStatusBarTexture("Interface\\AddOns\\!Tehtip\\Minimalist.tga")
local ManaBar = CreateFrame("StatusBar", nil, GameTooltipStatusBar)
ManaBar:SetStatusBarTexture("Interface\\AddOns\\!Tehtip\\Minimalist.tga")
ManaBar:SetHeight(8)
ManaBar:SetPoint("TOPLEFT", GameTooltip, "BOTTOMLEFT", 2, -10)
ManaBar:SetPoint("TOPRIGHT", GameTooltip, "BOTTOMRIGHT", -2, -10)

local classColours = {}
local factionColors = {}
local levelDifficultyColors = {}
for k,v in pairs(CUSTOM_CLASS_COLORS or RAID_CLASS_COLORS) do
	classColours[k] = format("%2x%2x%2x", v.r*255, v.g*255, v.b*255)
end
for k,v in pairs(FACTION_BAR_COLORS) do
	factionColors[k] = format("%2x%2x%2x", v.r*255, v.g*255, v.b*255)
end
for k,v in pairs(QuestDifficultyColors) do
	levelDifficultyColors[k] = format("%2x%2x%2x", v.r*255, v.g*255, v.b*255)
end

local function find(text, pattern)
	local start, stop = string.find(text, pattern)
	if start and stop then return string.sub(text, start, stop) end
end

local LEVEL_start = "^" .. (type(LEVEL) == "string" and LEVEL or "Level")
local function NPCSubtitle(unit)
	tehtip:ClearLines()
	tehtip:SetUnit(unit)
	if not tehtip:IsOwned(UIParent) then
		tehtip:SetOwner(UIParent, "ANCHOR_NONE")
	end
	local left_2 = tehtipTextLeft2:GetText()
	if not left_2 or find(left_2, LEVEL_start) then
		return nil
	end
	return left_2
end
local PVP = type(PVP) == "string" and PVP or "PvP"
local function NPCFaction(unit)
	tehtip:ClearLines()
	tehtip:SetUnit(unit)
	if not tehtip:IsOwned(UIParent) then
		tehtip:SetOwner(UIParent, "ANCHOR_NONE")
	end
	local left_2 = tehtipTextLeft2:GetText()
	local left_3 = tehtipTextLeft3:GetText()
	
	local hasGuild = not find(left_2, LEVEL_start)
	local factionText = not hasGuild and left_3 or tehtipTextLeft4:GetText()
	if factionText == PVP then
		return faction
	else
		return factionText
	end
end

local function colorUnit(unit)
	local color
	if UnitIsPlayer(unit)--[[ and not UnitCanAttack("player", unit) and not UnitCanAttack(unit, "player")]] then
		--[[if UnitIsPVP(unit) then
			color = factionColors[5]
		else]]
			local class = select(2, UnitClass(unit))
			color = classColours[class]
		--end
	elseif UnitPlayerControlled(unit) then
		if UnitCanAttack(unit, "player") then
			color = factionColors[1]
		elseif UnitCanAttack("player", unit) then
			color = factionColors[4]
		else
			color = "00ABFF"
		end
	else
		local reaction = UnitReaction(unit, "player")
		--if reaction == nil then print("TheTip Debug: "..unit) end
		if reaction then
			color = factionColors[reaction]
		end
	end
	return color or "FFFFFF"
end

local function levelDifficultyColor(level)
	local levelDiff = level - UnitLevel("player");
	local color;
	if ( levelDiff >= 5 ) then
		return levelDifficultyColors["impossible"];
	elseif ( levelDiff >= 3 ) then
		return levelDifficultyColors["verydifficult"];
	elseif ( levelDiff >= -2 ) then
		return levelDifficultyColors["difficult"];
	elseif ( -levelDiff <= GetQuestGreenRange() ) then
		return levelDifficultyColors["standard"];
	else
		return levelDifficultyColors["trivial"];
	end
end

function aceHook:formatTooltip(self)
	--add the lines!
	local line = 1
	local unitName, unit = self:GetUnit()
	--print(unit)
	if unit == nil then
		if UnitExists("mouseover") then
			unit = "mouseover"-- fix for mirror images
		else
			resetScripts()
		end
	end
	if UnitExists("mouseover") --[[or (UnitExists(unit) and not UnitInRange(unit))]] then
		local unitText
		local target
		local targetText
		local npcSubtitle
		local guild
		local factionText
		local unitColor = colorUnit(unit)
		local manaMax = UnitManaMax(unit)
		local mana = UnitMana(unit)
		ManaBar:SetMinMaxValues(0, manaMax)
		ManaBar:SetValue(mana)
		ManaBar:Show()
		-- ninjad from Cowtip
		local powerType = UnitPowerType(unit)
		if powerType == 0 then
			-- mana
			ManaBar:SetStatusBarColor(48/255, 113/255, 191/255)
		elseif powerType == 1 then
			-- rage
			ManaBar:SetStatusBarColor(226/255, 45/255, 75/255)
		elseif powerType == 2 then
			-- focus
			ManaBar:SetStatusBarColor(1, 210/255, 0)
		elseif powerType == 6 then
			-- runic power
			ManaBar:SetStatusBarColor(0.00, 0.82, 1.00)
		else
			-- energy
			ManaBar:SetStatusBarColor(1, 220/255, 25/255)
		end
		
		if UnitExists(format("%s%s", unit, "target")) then
			target = UnitName(format("%s%s", unit, "target"))
			if target then
				if UnitIsUnit(target, "player") then
					targetText = format(":%s", "|cFFFFFFFF<<YOU>>|r")
				else
					local color = colorUnit(format("%s%s", unit, "target"))
					targetText = format(":|cFF%s%s|r", color, target)
				end
			end
		else
			targetText = ""
		end
		unitText = format("|cFF%s%s|r%s", unitColor, unitName, targetText)
		
		local raceclass
		-- NPC!
		if not UnitIsPlayer(unit) and not UnitPlayerControlled(unit) then
			local title = NPCSubtitle(unit)
			local color = "00ABFF"
			if title then npcSubtitle = format("|cFF%s<%s>|r", color, title) end
			
			raceclass = format("|cFFE5B200%s|r", UnitCreatureType(unit))
			factionText = NPCFaction(unit)
		-- Pet!
		elseif (not UnitIsPlayer(unit) and UnitPlayerControlled(unit)) then
			local title = NPCSubtitle(unit)
			local color = "00ABFF"
			if title then npcSubtitle = format("|cFF%s<%s>|r", color, title) end
			
			local family = UnitCreatureFamily(unit)
			local creatureType = UnitCreatureType(unit)
			raceclass = format("|cFFE5B200%s|r", family or creatureType)
		-- Player!
		else
			local g = GetGuildInfo(unit)
			local color
			if UnitIsInMyGuild(unit) or friendlist[unitName] then
				color = "9400D4"
			else
				color = "00ABFF"
			end
			if g then guild = format("|cFF%s<%s>", color, g) end
			
			local race = UnitRace(unit)
			local eClass, class = UnitClass(unit)
			color = classColours[class] or "CC4C38"
			local coloredClass = format("|cFF%s%s|r", color, eClass)
			raceclass = format("%s|r %s", race, coloredClass)
		end
		local level = UnitLevel(unit)
		local levelColor = levelDifficultyColor(level)
		if level == -1 then level = "??" end
		local levelLine = format("|cFF%s%s |r|cFFE5B200%s", levelColor, level, raceclass)
		
		if unitText then
			_G["GameTooltipTextLeft"..1]:SetText(unitText)
			line = line + 1
		end
		if npcSubtitle then
			_G["GameTooltipTextLeft"..line]:SetText(npcSubtitle)
			line = line + 1
		end
		if guild then
			_G["GameTooltipTextLeft"..line]:SetText(guild)
			line = line + 1
		end
		if levelLine then
			_G["GameTooltipTextLeft"..line]:SetText(levelLine)
			line = line + 1
		end
		if factionText then
			_G["GameTooltipTextLeft"..line]:SetText(factionText)
			line = line + 1
		end
		
		-- background coloring
		local bR, bG, bB
		if (UnitIsDead(unit) and not UnitIsFeignDeath(unit)) or (UnitIsTapped(unit) and not UnitIsTappedByPlayer(unit)) then
			bR, bG, bB = 0.54, 0.54, 0.54
		elseif (not UnitIsPlayer(unit) and not UnitPlayerControlled(unit)) then
			local reaction = UnitReaction(unit, "player")
			bR, bG, bB = FACTION_BAR_COLORS[reaction].r, FACTION_BAR_COLORS[reaction].g, FACTION_BAR_COLORS[reaction].b
		elseif (UnitIsPlayer(unit) or UnitPlayerControlled(unit)) and (UnitCanAttack("player", unit) or UnitCanAttack(unit, "player")) then
			bR, bG, bB = FACTION_BAR_COLORS[3].r, FACTION_BAR_COLORS[3].g, FACTION_BAR_COLORS[3].b
		elseif (UnitIsPlayer(unit) or UnitPlayerControlled(unit)) and UnitCanAttack("player", unit) and UnitCanAttack(unit, "player") then
			bR, bG, bB = FACTION_BAR_COLORS[5].r, FACTION_BAR_COLORS[5].g, FACTION_BAR_COLORS[5].b
		else
			bR, bG, bB = 0.0, 0.0, 0.5
		end
		
		
		GameTooltipTextLeft1:SetFont("Fonts\\FRIZQT__.TTF", 14, "OUTLINE")
		GameTooltip:SetBackdropColor(bR, bG, bB)
		GameTooltip:Show()
	end
end

local function updateTip()
	aceHook:formatTooltip(GameTooltip)
end
local function scriptUpdateTip()
	TipUpdateFrame:SetScript("OnUpdate", updateTip)
end

aceHook:SecureHookScript(GameTooltip, "OnTooltipSetUnit", scriptUpdateTip)
aceHook:SecureHookScript(GameTooltip, "OnTooltipCleared", resetScripts)

local function FRIENDLIST_UPDATE()
	friendlist = {}
	for i=1, GetNumFriends() do
		local name = GetFriendInfo(i)
		friendlist[name] = name
	end
end
FRIENDLIST_UPDATE()

TipUpdateFrame:RegisterEvent("FRIENDLIST_UPDATE")
TipUpdateFrame:SetScript("OnEvent", FRIENDLIST_UPDATE)

local GameTooltip_OnUpdate_old = GameTooltip_OnUpdate
local function GameTooltip_OnUpdate_new(self, elapsed)
	GameTooltip_OnUpdate_old(self, elapsed)
	print("DebugTT")
end
GameTooltip_OnUpdate = GameTooltip_OnUpdate_new
